<?php
/**
 * Social Login integration EasySocial
 *
 * @version    1.0
 * @author        Arkadiy
 * @copyright    © 2012. All rights reserved.
 * @license    GNU/GPL v.3 or later.
 */

// No direct access
defined('_JEXEC') or die;
jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');
jimport('joomla.image.image');

class plgSlogin_integrationEasySocial extends JPlugin
{
    public function onAfterSloginLoginUser($user, $provider, $info)
    {
        $path 	= JPATH_ROOT . '/administrator/components/com_easysocial/includes/foundry.php';

        if( !JFile::exists( $path ) )
        {
            return true;
        }

        // Include the foundry engine
        require_once( $path );

        $easySocialUser = $this->getUser($user->id);

        if(empty($easySocialUser->id))
        {
            return true;
        }

        if (!$this->checkUserFields($user->id))
        {
            $this->createUserFields($user->id);
        }

        if (!isset($easySocialUser->avatars) || empty($easySocialUser->avatars['small']))
        {
            $this->setAvatar($easySocialUser);
        }

    }

    private function getUser($userId)
    {
        return FD::user($userId);
    }

    private function createUserFields($userId)
    {
        if (JPluginHelper::isEnabled('slogin_integration', 'profile'))
        {
            include_once JPATH_ROOT . '/plugins/slogin_integration/profile/helper.php';
        }
        else
        {
            return false;
        }

        $profile = plgProfileHelper::getProfile($userId);
        if ($profile === false)
        {
            return false;
        }

        $db = JFactory::getDBO();

        $nameFieldId = $this->params->get('name', 0);
        $birthdayFieldId = $this->params->get('birthday', 0);
        $genderFieldId = $this->params->get('gender', 0);
        $addressFieldId = $this->params->get('address', 0);

        $fields = array();

        if($nameFieldId && $profile->f_name)
        {
            $fields[] = array(
                'field_id' => $nameFieldId,
                'type' => 'user',
                'datakey' => 'first',
                'data' => $profile->f_name,
                'raw' => $profile->f_name
            );
        }

        if($nameFieldId && $profile->l_name)
        {
            $fields[] = array(
                'field_id' => $nameFieldId,
                'type' => 'user',
                'datakey' => 'last',
                'data' => $profile->l_name,
                'raw' => $profile->l_name
            );
        }

        if($nameFieldId && ($profile->f_name || $profile->l_name))
        {
            $name = $profile->f_name.' '.$profile->l_name;
            $fields[] = array(
                'field_id' => $nameFieldId,
                'type' => 'user',
                'datakey' => 'name',
                'data' => $name,
                'raw' => $name
            );
        }

        if($birthdayFieldId && !empty($profile->birthday))
        {
            $fields[] = array(
                'field_id' => $birthdayFieldId,
                'type' => 'user',
                'datakey' => 'date',
                'data' => $profile->birthday,
                'raw' => $profile->birthday
            );
        }

        if($genderFieldId)
        {
            $gender = ((int)$profile->gender == 0) ? 2 : 1;
            $fields[] = array(
                'field_id' => $genderFieldId,
                'type' => 'user',
                'datakey' => '',
                'data' => $gender,
                'raw' => $gender
            );
        }

        if($addressFieldId && $profile->lat && $profile->lng)
        {
            $fields[] = array(
                'field_id' => $addressFieldId,
                'type' => 'user',
                'datakey' => 'latitude',
                'data' => $profile->lat,
                'raw' => $profile->lat
            );

            $fields[] = array(
                'field_id' => $addressFieldId,
                'type' => 'user',
                'datakey' => 'longitude',
                'data' => $profile->lng,
                'raw' => $profile->lng
            );
        }

        $errors = 0;

        foreach($fields as $field)
        {
            $row = new stdclass();
            $row->uid = $userId;
            $row->field_id = $field['field_id'];
            $row->type = $field['type'];
            $row->datakey = $field['datakey'];
            $row->data = $field['data'];
            $row->params = '';
            $row->raw = $field['raw'];

            if (!$db->insertObject('#__social_fields_data', $row, 'id'))
            {
                $errors++;
            }

        }

        return ($errors>0) ? false : true;
    }




    private function setAvatar($user)
    {
        if (JPluginHelper::isEnabled('slogin_integration', 'profile'))
        {
            include_once JPATH_ROOT . '/plugins/slogin_integration/profile/helper.php';
        }
        else
        {
            return false;
        }

        include_once JPATH_ROOT . '/administrator/components/com_easysocial/constants.php';

        $profile = plgProfileHelper::getProfile($user->id);

        if(empty($profile->avatar))
        {
            return false;
        }

        $tmpAvatarFile 		= JPATH_ROOT.'/'.$profile->avatar;

        $image = FD::image();
        $image->load($tmpAvatarFile);

        $avatar		= FD::avatar($image, $user->id, SOCIAL_TYPE_USER);

        // Check if there's a profile photos album that already exists.
        $albumModel	= FD::model('Albums');

        // Retrieve the user's default album
        $album		= $albumModel->getDefaultAlbum($user->id, SOCIAL_TYPE_USER, SOCIAL_ALBUM_PROFILE_PHOTOS);

        $photo 				= FD::table('Photo');
        $photo->uid 		= $user->id;
        $photo->user_id 	= $user->id;
        $photo->type 		= SOCIAL_TYPE_USER;
        $photo->album_id 	= $album->id;
        $photo->title 		= $user->getName();
        $photo->caption 	= JText::_('COM_EASYSOCIAL_PHOTO_IMPORTED_FROM_FACEBOOK');
        $photo->ordering	= 0;

        // We need to set the photo state to "SOCIAL_PHOTOS_STATE_TMP"
        $photo->state 		= SOCIAL_PHOTOS_STATE_TMP;

        // Try to store the photo first
        $state 		= $photo->store();

        if (!$state) {
            $this->setError(JText::_('PLG_FIELDS_AVATAR_ERROR_CREATING_PHOTO_OBJECT'));
            return false;
        }

        // Push all the ordering of the photo down
        $photosModel = FD::model('photos');
        $photosModel->pushPhotosOrdering($album->id, $photo->id);

        // If album doesn't have a cover, set the current photo as the cover.
        if (!$album->hasCover()) {
            $album->cover_id 	= $photo->id;

            // Store the album
            $album->store();
        }

        // Get the photos library
        $photoLib 	= FD::get('Photos', $image);
        $storage   = $photoLib->getStoragePath($album->id, $photo->id);
        $paths 		= $photoLib->create($storage);

        // Create metadata about the photos
        foreach ($paths as $type => $fileName) {
            $meta 				= FD::table('PhotoMeta');
            $meta->photo_id		= $photo->id;
            $meta->group 		= SOCIAL_PHOTOS_META_PATH;
            $meta->property 	= $type;
            $meta->value		= $storage . '/' . $fileName;

            $meta->store();
        }

        // Assign a badge for the user
        $photo->assignBadge('photos.upload', $user->id);

        // @points: photos.upload
        // Assign points when user uploads a new photo
        $photo->assignPoints('photos.upload', $user->id);

        // Synchronize Indexer
        $indexer 	= FD::get('Indexer');
        $template	= $indexer->getTemplate();
        $template->setContent($photo->title, $photo->caption);

        // $url 	= FRoute::photos(array('layout' => 'item', 'id' => $photo->getAlias()));
        $url 	= $photo->getPermalink();
        $url 	= '/' . ltrim($url, '/');
        $url 	= str_replace('/administrator/', '/', $url);

        $template->setSource($photo->id, SOCIAL_INDEXER_TYPE_PHOTOS, $photo->uid, $url);
        $template->setThumbnail($photo->getSource('thumbnail'));

        $indexer->index($template);

        // Create the avatars now
        $avatar->store($photo);
    }

    private function getUserFields($userId)
    {
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);

        $query->select('*')
            ->from('#__social_fields_data')
            ->where('`uid` ='.$db->quote($userId));
        return $db->setQuery($query)->loadObjectList();
    }

    private function checkUserFields($userId)
    {
        $fields = $this->getUserFields($userId);
        return (is_array($fields) && count($fields)) ? true : false;
    }
}

